/*Здесь программа крутится в цикле пока не нажали кнопку потом проирывает sound.wav и обратно в цикл опрашивать кнопку---------------------------------------------------------------*/
/* 8-pin SD audio player R0.04                     (C)ChaN, 2010 */
/*---------------------------------------------------------------*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/wdt.h>
#include <string.h>
#include "diskio.h"
#include "pff.h"

#ifndef MODE
#error Wrong make file.
#endif
#define FCC(c1,c2,c3,c4)	(((DWORD)c4<<24)+((DWORD)c3<<16)+((WORD)c2<<8)+(BYTE)c1)	/* FourCC */


/*---------------------------------------------------------*/
/* Work Area                                               */
/*---------------------------------------------------------*/

volatile BYTE FifoRi, FifoWi, FifoCt;	/* FIFO controls */

BYTE Buff[256];		/* Wave output FIFO */

FATFS Fs;			/* File system object */
DIR Dir;			/* Directory object */
FILINFO Fno;		/* File information */

WORD rb;			/* Return value. Put this here to avoid bugs of avr-gcc */



EMPTY_INTERRUPT(WDT_vect);


/*---------------------------------------------------------*/

static
DWORD load_header (void)	/* 0:Invalid format, 1:I/O error, >1:Number of samples */
{
	DWORD sz;


	if (pf_read(Buff, 12, &rb)) return 1;	/* Load file header (12 bytes) */

	if (rb != 12 || LD_DWORD(Buff+8) != FCC('W','A','V','E')) return 0;

	for (;;) {
		pf_read(Buff, 8, &rb);			/* Get Chunk ID and size */
		if (rb != 8) return 0;
		sz = LD_DWORD(&Buff[4]);		/* Chunk size */

		switch (LD_DWORD(&Buff[0])) {	/* FCC */
		case FCC('f','m','t',' ') :					/* 'fmt ' chunk */
			if (sz > 100 || sz < 16) return 0;		/* Check chunk size */
			pf_read(Buff, sz, &rb);					/* Get content */
			if (rb != sz) return 0;
			if (Buff[0] != 1) return 0;				/* Check coding type (1) */
			if (Buff[2] != 1 && Buff[2] != 2) 		/* Check channels (1/2) */
				return 0;
			GPIOR0 = Buff[2];						/* Get channel flag */
			if (Buff[14] != 8 && Buff[14] != 16)	/* Check resolution (8/16) */
				return 0;
			GPIOR0 |= Buff[14];		/* Resolution flag */
			OCR0A = (BYTE)(F_CPU/8/LD_WORD(&Buff[4]))-1;	/* Sampling freq */
			break;

		case FCC('d','a','t','a') :				/* 'data' chunk (start to play) */
			return sz;

		case FCC('L','I','S','T') :				/* 'LIST' chunk (skip) */
		case FCC('f','a','c','t') :				/* 'fact' chunk (skip) */
			pf_lseek(Fs.fptr + sz);
			break;

		default :								/* Unknown chunk (error) */
			return 0;
		}
	}

	return 0;
}



static
UINT play (
	const char *fn
)
{
	DWORD sz;
	FRESULT res;
	BYTE sw;
	WORD btr;


	if ((res = pf_open(fn)) == FR_OK) {
		sz = load_header();			/* Load file header */
		if (sz < 256) return (UINT)sz;

		if (!TCCR1) {				/* Enable audio out if not enabled */
			PLLCSR = 0b00000110;	/* Select PLL clock for TC1.ck */
			GTCCR =  0b01100000;	/* Enable TC1.OCB as PWM out (L-ch) */
			OCR1B = 128; OCR1A = 128;
			TCCR1 = MODE ? 0b01100001 : 0b00000001;	/* Start TC1 with TC1.OCA is enabled as PWM out (R-ch) */
			TCCR0A = 0b00000010;	/* Enable TC0.ck = 2MHz as interval timer */
			TCCR0B = 0b00000010;
			TIMSK = _BV(OCIE0A);
		}

		FifoCt = 0; FifoRi = 0; FifoWi = 0;		/* Reset FIFO */
		pf_read(0, 512 - (Fs.fptr % 512), &rb);	/* Snip sector unaligned part */
		sz -= rb;

		sw = 1;		/* Button status flag */
		do {
			/* Forward audio data */
			btr = (sz > 1024) ? 1024 : (WORD)sz;
			res = pf_read(0, btr, &rb);
			if (res != FR_OK || btr != rb) break;
			sz -= rb;
			/* Check button down and break on button down */
			sw <<= 1;
			wdt_reset();
		} while (rb == 1024);	/* Repeat until all data read */
	}

	while (FifoCt) ;			/* Wait for FIFO empty */
	OCR1A = 128; OCR1B = 128;

	return res;
}



static
void delay500 (void)
{
	TCCR1 = 0; GTCCR = 0;	/* Stop TC1 */
	TCCR0B = 0; TCCR0A = 0;	/* Stop TC0 */

	wdt_reset();
	WDTCR = _BV(WDE) | _BV(WDIE) | 0b101;	/* Enable WDT interrupt in timeout of 0.5s */
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);	/* Enter power down mode */
	sleep_mode();

	WDTCR = _BV(WDE) | 0b110;				/* Enable WDT reset in timeout of 1s */
}



/*-----------------------------------------------------------------------*/
/* Main                                                                  */


int main (void)
{
	MCUSR = 0;
	WDTCR = _BV(WDE) | 0b110;	/* Enable WDT reset in timeout of 1s */

	PORTB = 0b111011;		/* Initialize port: - - H H H L H P */
	DDRB  = 0b111110;

	sei();

	pf_mount(&Fs);	/* Initialize FS */
	for (;;) {

			if (bit_is_clear(PINB, 0) == 1) play('sound.wav');
			
		}
	
}



